"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ColorSchema = exports.UserSchema = exports.CommentSchema = exports.RawCommentSchema = exports.AttachmentSchema = exports.LabelSchema = exports.SectionSchema = exports.ProjectSchema = exports.RawProjectSchema = exports.TaskSchema = exports.RawTaskSchema = exports.DeadlineSchema = exports.DurationSchema = exports.DueDateSchema = void 0;
var zod_1 = require("zod");
exports.DueDateSchema = zod_1.z
    .object({
    isRecurring: zod_1.z.boolean(),
    string: zod_1.z.string(),
    date: zod_1.z.string(),
})
    .extend({
    datetime: zod_1.z.string().nullable().optional(),
    timezone: zod_1.z.string().nullable().optional(),
    lang: zod_1.z.string().nullable().optional(),
});
exports.DurationSchema = zod_1.z.object({
    amount: zod_1.z.number().positive('Value should be greater than zero'),
    unit: zod_1.z.enum(['minute', 'day']),
});
exports.DeadlineSchema = zod_1.z.object({
    date: zod_1.z.string(),
    lang: zod_1.z.string(),
});
exports.RawTaskSchema = zod_1.z.object({
    userId: zod_1.z.string(),
    id: zod_1.z.string(),
    projectId: zod_1.z.string(),
    sectionId: zod_1.z.string().nullable(),
    parentId: zod_1.z.string().nullable(),
    addedByUid: zod_1.z.string(),
    assignedByUid: zod_1.z.string().nullable(),
    responsibleUid: zod_1.z.string().nullable(),
    labels: zod_1.z.array(zod_1.z.string()),
    deadline: exports.DeadlineSchema.nullable(),
    duration: exports.DurationSchema.nullable(),
    checked: zod_1.z.boolean(),
    isDeleted: zod_1.z.boolean(),
    addedAt: zod_1.z.string(),
    completedAt: zod_1.z.string().nullable(),
    updatedAt: zod_1.z.string(),
    due: exports.DueDateSchema.nullable(),
    priority: zod_1.z.number().int(),
    childOrder: zod_1.z.number().int(),
    content: zod_1.z.string(),
    description: zod_1.z.string(),
    dayOrder: zod_1.z.number().int(),
    isCollapsed: zod_1.z.boolean(),
});
exports.TaskSchema = zod_1.z.object({
    id: zod_1.z.string(),
    assignerId: zod_1.z.string().nullable(),
    assigneeId: zod_1.z.string().nullable(),
    projectId: zod_1.z.string(),
    sectionId: zod_1.z.string().nullable(),
    parentId: zod_1.z.string().nullable(),
    order: zod_1.z.number().int(),
    content: zod_1.z.string(),
    description: zod_1.z.string(),
    isCompleted: zod_1.z.boolean(),
    labels: zod_1.z.array(zod_1.z.string()),
    priority: zod_1.z.number().int(),
    creatorId: zod_1.z.string(),
    createdAt: zod_1.z.string(),
    due: exports.DueDateSchema.nullable(),
    url: zod_1.z.string(),
    duration: exports.DurationSchema.nullable(),
    deadline: exports.DeadlineSchema.nullable(),
});
exports.RawProjectSchema = zod_1.z.object({
    id: zod_1.z.string(),
    canAssignTasks: zod_1.z.boolean(),
    childOrder: zod_1.z.number().int().nullable(),
    color: zod_1.z.string(),
    createdAt: zod_1.z.string(),
    isArchived: zod_1.z.boolean(),
    isDeleted: zod_1.z.boolean(),
    isFavorite: zod_1.z.boolean(),
    isFrozen: zod_1.z.boolean(),
    name: zod_1.z.string(),
    updatedAt: zod_1.z.string(),
    viewStyle: zod_1.z.string(),
    defaultOrder: zod_1.z.number().int().nullable(),
    description: zod_1.z.string(),
    publicAccess: zod_1.z.boolean(),
    parentId: zod_1.z.string().nullable().optional(),
    inboxProject: zod_1.z.boolean().optional(),
    isCollapsed: zod_1.z.boolean(),
    isShared: zod_1.z.boolean(),
});
exports.ProjectSchema = zod_1.z.object({
    id: zod_1.z.string(),
    parentId: zod_1.z.string().nullable(),
    order: zod_1.z.number().int().nullable(),
    color: zod_1.z.string(),
    name: zod_1.z.string(),
    isShared: zod_1.z.boolean(),
    isFavorite: zod_1.z.boolean(),
    isInboxProject: zod_1.z.boolean(),
    isTeamInbox: zod_1.z.boolean(),
    url: zod_1.z.string(),
    viewStyle: zod_1.z.string(),
});
exports.SectionSchema = zod_1.z.object({
    id: zod_1.z.string(),
    userId: zod_1.z.string(),
    projectId: zod_1.z.string(),
    addedAt: zod_1.z.string(),
    updatedAt: zod_1.z.string(),
    archivedAt: zod_1.z.string().nullable(),
    name: zod_1.z.string(),
    sectionOrder: zod_1.z.number().int(),
    isArchived: zod_1.z.boolean(),
    isDeleted: zod_1.z.boolean(),
    isCollapsed: zod_1.z.boolean(),
});
exports.LabelSchema = zod_1.z.object({
    id: zod_1.z.string(),
    order: zod_1.z.number().int().nullable(),
    name: zod_1.z.string(),
    color: zod_1.z.string(),
    isFavorite: zod_1.z.boolean(),
});
exports.AttachmentSchema = zod_1.z
    .object({
    resourceType: zod_1.z.string(),
})
    .extend({
    fileName: zod_1.z.string().nullable().optional(),
    fileSize: zod_1.z.number().int().nullable().optional(),
    fileType: zod_1.z.string().nullable().optional(),
    fileUrl: zod_1.z.string().nullable().optional(),
    fileDuration: zod_1.z.number().int().nullable().optional(),
    uploadState: zod_1.z.enum(['pending', 'completed']).nullable().optional(),
    image: zod_1.z.string().nullable().optional(),
    imageWidth: zod_1.z.number().int().nullable().optional(),
    imageHeight: zod_1.z.number().int().nullable().optional(),
    url: zod_1.z.string().nullable().optional(),
    title: zod_1.z.string().nullable().optional(),
});
exports.RawCommentSchema = zod_1.z
    .object({
    id: zod_1.z.string(),
    itemId: zod_1.z.string().optional(),
    projectId: zod_1.z.string().optional(),
    content: zod_1.z.string(),
    postedAt: zod_1.z.string(),
    fileAttachment: exports.AttachmentSchema.nullable(),
    postedUid: zod_1.z.string(),
    uidsToNotify: zod_1.z.array(zod_1.z.string()).nullable(),
    reactions: zod_1.z.record(zod_1.z.string(), zod_1.z.array(zod_1.z.string())).nullable(),
    isDeleted: zod_1.z.boolean(),
})
    .refine(function (data) {
    // At least one of itemId or projectId must be present
    return ((data.itemId !== undefined && data.projectId === undefined) ||
        (data.itemId === undefined && data.projectId !== undefined));
}, {
    message: 'Exactly one of itemId or projectId must be provided',
});
exports.CommentSchema = exports.RawCommentSchema.transform(function (data) {
    var itemId = data.itemId, rest = __rest(data, ["itemId"]);
    return __assign(__assign({}, rest), { 
        // Map itemId to taskId for backwards compatibility
        taskId: itemId });
});
exports.UserSchema = zod_1.z.object({
    id: zod_1.z.string(),
    name: zod_1.z.string(),
    email: zod_1.z.string(),
});
exports.ColorSchema = zod_1.z.object({
    /** @deprecated No longer used */
    id: zod_1.z.number(),
    /** The key of the color (i.e. 'berry_red') */
    key: zod_1.z.string(),
    /** The display name of the color (i.e. 'Berry Red') */
    displayName: zod_1.z.string(),
    /** @deprecated Use {@link Color.displayName} instead */
    name: zod_1.z.string(),
    /** The hex value of the color (i.e. '#b8255f') */
    hexValue: zod_1.z.string(),
    /**
     * @deprecated Use {@link Color.hexValue} instead
     */
    value: zod_1.z.string(),
});
